@@ -35,7 +35,6 @@ static GMainLoop *loop = NULL;
 static GTimer *log_timer;
 static int log_fd = -1;
 static gboolean debug = FALSE;
-static gboolean stopping = FALSE;
 
 static DisplayManager *display_manager = NULL;
 static XDMCPServer *xdmcp_server = NULL;
@@ -62,55 +61,54 @@ typedef struct
 #define LIGHTDM_BUS_NAME "org.freedesktop.DisplayManager"
 
 static void
-log_cb (const gchar *log_domain, GLogLevelFlags log_level,
-        const gchar *message, gpointer data)
+log_cb (const gchar *log_domain, GLogLevelFlags log_level, const gchar *message, gpointer data)
 {
+    const gchar *prefix;
+    gchar *text;
+
+    switch (log_level & G_LOG_LEVEL_MASK)
+    {
+    case G_LOG_LEVEL_ERROR:
+        prefix = "ERROR:";
+        break;
+    case G_LOG_LEVEL_CRITICAL:
+        prefix = "CRITICAL:";
+        break;
+    case G_LOG_LEVEL_WARNING:
+        prefix = "WARNING:";
+        break;
+    case G_LOG_LEVEL_MESSAGE:
+        prefix = "MESSAGE:";
+        break;
+    case G_LOG_LEVEL_INFO:
+        prefix = "INFO:";
+        break;
+    case G_LOG_LEVEL_DEBUG:
+        prefix = "DEBUG:";
+        break;
+    default:
+        prefix = "LOG:";
+        break;
+    }
+
+    text = g_strdup_printf ("[%+.2fs] %s %s\n", g_timer_elapsed (log_timer, NULL), prefix, message);
+
     /* Log everything to a file */
     if (log_fd >= 0)
     {
-        const gchar *prefix;
-        gchar *text;
         ssize_t n_written;
-
-        switch (log_level & G_LOG_LEVEL_MASK)
-        {
-        case G_LOG_LEVEL_ERROR:
-            prefix = "ERROR:";
-            break;
-        case G_LOG_LEVEL_CRITICAL:
-            prefix = "CRITICAL:";
-            break;
-        case G_LOG_LEVEL_WARNING:
-            prefix = "WARNING:";
-            break;
-        case G_LOG_LEVEL_MESSAGE:
-            prefix = "MESSAGE:";
-            break;
-        case G_LOG_LEVEL_INFO:
-            prefix = "INFO:";
-            break;
-        case G_LOG_LEVEL_DEBUG:
-            prefix = "DEBUG:";
-            break;
-        default:
-            prefix = "LOG:";
-            break;
-        }
-
-        text = g_strdup_printf ("[%+.2fs] %s %s\n", g_timer_elapsed (log_timer, NULL), prefix, message);
         n_written = write (log_fd, text, strlen (text));
         if (n_written < 0)
             ; /* Check result so compiler doesn't warn about it */
-        g_free (text);
     }
 
-    /* Only show debug if requested */
-    if (log_level & G_LOG_LEVEL_DEBUG) {
-        if (debug)
-            g_log_default_handler (log_domain, log_level, message, data);
-    }
+    /* Log to stderr if requested */
+    if (debug)
+        g_printerr ("%s", text);
     else
-        g_log_default_handler (log_domain, log_level, message, data);    
+        g_log_default_handler (log_domain, log_level, message, data);
+
+    g_free (text);
 }
 
 static void
@@ -126,6 +124,7 @@ log_init (void)
     g_free (log_dir);
 
     log_fd = open (path, O_WRONLY | O_CREAT | O_TRUNC, 0600);
+    fcntl (log_fd, F_SETFD, FD_CLOEXEC);
     g_log_set_default_handler (log_cb, NULL);
 
     g_debug ("Logging to %s", path);
@@ -143,19 +142,6 @@ signal_cb (Process *process, int signum)
 static gboolean
 exit_cb (gpointer data)
 {
-    exit (exit_code);
-}
-
-static void
-display_manager_stopped_cb (DisplayManager *display_manager)
-{
-    g_debug ("Stopping Light Display Manager");
-
-    /* Mark as stopping because bus_acquired_cb can be called after this function, and even
-     * more weirdly display_manager is not NULL even though the following lines set it to
-     * be! */
-    stopping = TRUE;
-
     /* Clean up display manager */
     g_object_unref (display_manager);
     display_manager = NULL;
@@ -167,6 +153,14 @@ display_manager_stopped_cb (DisplayManager *display_manager)
     if (session_bus_entries)
         g_hash_table_unref (session_bus_entries);
 
+    g_debug ("Exiting with return value %d", exit_code);
+    exit (exit_code);
+}
+
+static void
+display_manager_stopped_cb (DisplayManager *display_manager)
+{
+    g_debug ("Stopping daemon");
     g_idle_add (exit_cb, NULL);
 }
 
@@ -588,6 +582,7 @@ start_session_cb (Display *display, Seat *seat)
 static void
 session_stopped_cb (Session *session, Seat *seat)
 {
+    g_signal_handlers_disconnect_matched (session, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, seat);
     g_hash_table_remove (session_bus_entries, session);
 }
 
@@ -679,11 +674,14 @@ seat_added_cb (DisplayManager *display_manager, Seat *seat)
 static void
 seat_removed_cb (DisplayManager *display_manager, Seat *seat)
 {
+    gboolean exit_on_failure;
+
+    exit_on_failure = seat_get_boolean_property (seat, "exit-on-failure");
     g_hash_table_remove (seat_bus_entries, seat);
 
-    if (seat_get_boolean_property (seat, "exit-on-failure"))
+    if (exit_on_failure)
     {
-        g_debug ("Stopping lightdm, required seat has stopped");
+        g_debug ("Required seat has stopped");
         exit_code = EXIT_FAILURE;
         display_manager_stop (display_manager);
     }
@@ -761,9 +759,6 @@ bus_acquired_cb (GDBusConnection *connection,
     GDBusNodeInfo *display_manager_info;
     GList *link;
 
-    if (stopping)
-        return;
-
     g_debug ("Acquired bus name %s", name);
 
     bus = connection;
@@ -889,7 +884,7 @@ main (int argc, char **argv)
           N_("Run as unprivileged user, skipping things that require root access"), NULL },
         { "passwd-file", 0, 0, G_OPTION_ARG_STRING, &passwd_path,
           /* Help string for command line --use-passwd flag */
-          N_("Use the given password file for authentication (for testing, requires --no-root)"), "FILE" },
+          N_("Use the given password file for authentication (for testing, requires --test-mode)"), "FILE" },
         { "pid-file", 0, 0, G_OPTION_ARG_STRING, &pid_path,
           /* Help string for command line --pid-file flag */
           N_("File to write PID into"), "FILE" },
@@ -932,7 +927,7 @@ main (int argc, char **argv)
         { NULL }
     };
     GError *error = NULL;
-
+ 
     g_thread_init (NULL);
     g_type_init ();
     loop = g_main_loop_new (NULL, FALSE);
@@ -999,7 +994,7 @@ main (int argc, char **argv)
     /* Don't allow to be run as root and use a password file (asking for danger!) */
     if (getuid () == 0 && passwd_path)
     {
-        g_printerr ("Only allowed to use --passwd-file when running with --no-root.\n"); 
+        g_printerr ("Only allowed to use --passwd-file when running with --test-mode.\n"); 
         return EXIT_FAILURE;
     }
 
